@@ -88,21 +88,6 @@ extern "C" {
  * Definitions that may not always be provided directly by the system
  *============================================================================*/
 
-/*
-  Force LARGEFILE_SOURCE if largefiles enabled under Linux or Blue Gene/L
-  (otherwise, we may encounter bugs with glibc 2.3 due to fseeko end ftello
-  not being correctly defined). Compiling with -D_GNU_SOURCE instead
-  of -D_POSIX_C_SOURCE=200112L seems to be another way to solve the problem.
-*/
-
-#if defined(bft_OS_Linux) || defined(__blrts__) || defined(__bgp__)
-# if defined(_FILE_OFFSET_BITS) && !defined(_GNU_SOURCE)
-#  if (_FILE_OFFSET_BITS == 64) && !defined(_LARGEFILE_SOURCE)
-#   define _LARGEFILE_SOURCE 1
-#  endif
-# endif
-#endif
-
 /*
  * Obtain definitions such as that of size_t through stddef.h (C99 standard)
  * if available (preferred method), or through stdlib.h (which defines
