@@ -6,7 +6,7 @@
   This file is part of the "Base Functions and Types" library, intended to
   simplify and enhance portability, memory and I/O use for scientific codes.
 
-  Copyright (C) 2004-2007  EDF
+  Copyright (C) 2004-2009  EDF
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -23,10 +23,30 @@
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-/*-----------------------------------------------------------------------------*/
+/*----------------------------------------------------------------------------*/
 
 #include "bft_config_defs.h"
 
+/*
+  Force LARGEFILE_SOURCE if largefiles enabled under 32-bit Linux or Blue Gene
+  (otherwise, we may encounter bugs with glibc 2.3 due to fseeko end ftello
+  not being correctly defined). Compiling with -D_GNU_SOURCE instead
+  of -D_POSIX_C_SOURCE=200112L seems to be another way to solve the problem.
+*/
+
+#if (SIZEOF_LONG < 8) && (_FILE_OFFSET_BITS == 64)
+# if defined(__linux__) || defined(__blrts__) || defined(__bgp__)
+#  if !defined(_POSIX_SOURCE)
+#    define _GNU_SOURCE 1
+#  endif
+#  if !defined(_GNU_SOURCE) && !defined(_LARGEFILE_SOURCE)
+#   define _LARGEFILE_SOURCE 1
+#  endif
+# endif
+#endif
+
+/*-----------------------------------------------------------------------------*/
+
 /*
  * Standard C library headers
  */
@@ -113,7 +133,7 @@ struct _bft_file_t {
 #  if (BFT_SIZEOF_Z_OFF_T == BFT_SIZEOF_LONG)
 typedef long bft_z_off_t;
 #  elif defined (HAVE_LONG_LONG)
-#    if (BFT_SIZEOF_Z_OFF_T == BFT_SIZEOF_LONG)
+#    if (BFT_SIZEOF_Z_OFF_T == BFT_SIZEOF_LONG_LONG)
 typedef long long bft_z_off_t;
 #    else
 #      error "z_off_t returned by zlibCompileFlags() neither long nor long long"
@@ -920,19 +940,33 @@ bft_file_tell(bft_file_t  *const f)
   assert(f != NULL);
 
   if (f->ptr != NULL) {
-#if defined (BFT_SIZEOF_OFF_T)
-#  if (BFT_SIZEOF_OFF_T > BFT_SIZEOF_LONG)
-#    if (HAVE_FSEEKO)
+
+#if (SIZEOF_LONG < 8)
+
+    /* For 32-bit systems, large file support might be necessary */
+
+# if defined(HAVE_FSEEKO) && (_FILE_OFFSET_BITS == 64)
     offset = (bft_file_off_t)ftello(f->ptr);
-#    else
-#      error "sizeof(off_t) > sizeof(long) but fseeko()/ftello() not available"
-#    endif
-#  else
-    offset = (bft_file_off_t)ftell(f->ptr);
-#  endif
-#else
+# else
+    /*
+      Without ftello, ftell will fail above 2 Gigabytes, in which case
+      offset == -1 and errno == EOVERFLOW, but should work on smaller
+      files. We prefer not to be too strict about fseeko availability, as
+      the only 32-bit case without ftello we have encountered is Cygwin
+      (for which ftello requires additional non-default libraries), which
+      is expected to be used mainly for small cases.
+    */
     offset = (bft_file_off_t)ftell(f->ptr);
 #endif
+
+#else /* SIZEOF_LONG >= 8) */
+
+    /* For 64-bit systems, standard ftell should be enough */
+
+    offset = (bft_file_off_t)ftell(f->ptr);
+
+#endif
+
   }
 
 #if defined(HAVE_ZLIB)
@@ -944,7 +978,7 @@ bft_file_tell(bft_file_t  *const f)
 
   if (offset < 0)
     _bft_file_error(__FILE__, __LINE__, 0,
-                    _("Error obtaining position in file \"%s\":\n"),
+                    _("Error obtaining position in file \"%s\":\n\n  %s"),
                     f->name, _bft_file_error_string(f));
 
   return offset;
@@ -965,19 +999,31 @@ bft_file_rewind(bft_file_t  *const f)
   assert(f != NULL);
 
   if (f->ptr != NULL) {
-#if defined (BFT_SIZEOF_OFF_T)
-#  if (BFT_SIZEOF_OFF_T > BFT_SIZEOF_LONG)
-#    if (HAVE_FSEEKO)
-    (void)fseeko(f->ptr, 0L, SEEK_SET);
-#    else
-#      error "sizeof(off_t) > sizeof(long) but fseeko()/ftello() not available"
-#    endif
-#  else
-    (void)fseek(f->ptr, 0L, SEEK_SET);
-#  endif
-#else
-    (void)fseek(f->ptr, 0L, SEEK_SET);
+
+    int retval = 0;
+
+#if (SIZEOF_LONG < 8)
+
+    /* For 32-bit systems, large file support might be necessary */
+
+# if defined(HAVE_FSEEKO) && (_FILE_OFFSET_BITS == 64)
+    retval = fseeko(f->ptr, 0L, SEEK_SET);
+# else
+    retval = fseek(f->ptr, 0L, SEEK_SET);
+# endif
+
+#else /* SIZEOF_LONG >= 8) */
+
+    /* For 64-bit systems, standard fseek should be enough */
+
+    retval = fseek(f->ptr, 0L, SEEK_SET);
+
 #endif
+
+    if (retval != 0)
+      _bft_file_error(__FILE__, __LINE__, errno,
+                      _("Error rewinding file \"%s\":\n\n  %s"),
+                      f->name, _bft_file_error_string(f));
   }
 
 #if defined(HAVE_ZLIB)
@@ -1041,25 +1087,55 @@ bft_file_seek(bft_file_t             *const f,
 
   if (f->ptr != NULL) {
 
-#if defined (BFT_SIZEOF_OFF_T)
-#  if (BFT_SIZEOF_OFF_T > BFT_SIZEOF_LONG)
-#    if (HAVE_FSEEKO)
+#if (SIZEOF_LONG < 8)
+
+    /* For 32-bit systems, large file support might be necessary */
+
+# if defined(HAVE_FSEEKO) && (_FILE_OFFSET_BITS == 64)
+
     retval = fseeko(f->ptr, (off_t)offset, _whence);
-#    else
-#      error "sizeof(off_t) > sizeof(long) but fseeko()/ftello() not available"
-#    endif
-#  else
-    retval = fseek(f->ptr, (long)offset, _whence);
-#  endif
-#else
-    retval = fseek(f->ptr, (long)offset, _whence);
-#endif
 
     if (retval != 0)
       _bft_file_error(__FILE__, __LINE__, errno,
-                      _("Error setting position in file \"%s\":\n"),
+                      _("Error setting position in file \"%s\":\n\n  %s"),
                       f->name, _bft_file_error_string(f));
 
+# else
+
+    /* Test if offset larger than allowed */
+
+    long _offset = offset;
+
+    if (_offset == offset) {
+      retval = fseek(f->sh, (long)offset, _whence);
+      if (retval != 0)
+        _bft_file_error(__FILE__, __LINE__, errno,
+                        _("Error setting position in file \"%s\":\n\n  %s"),
+                        f->name, _bft_file_error_string(f));
+    }
+    else {
+      retval = -1;
+      _bft_file_error
+        (__FILE__, __LINE__, errno,
+         _("Error setting position in file \"%s\":\n\n  %s"),
+         f->name,
+         _("sizeof(off_t) > sizeof(long) but fseeko() not available"));
+    }
+
+# endif /* defined(HAVE_FSEEKO) && (_FILE_OFFSET_BITS == 64) */
+
+#else /* SIZEOF_LONG >= 8) */
+
+    /* For 64-bit systems, standard fseek should be enough */
+
+    retval = fseek(f->ptr, (long)offset, _whence);
+
+    if (retval != 0)
+      _bft_file_error(__FILE__, __LINE__, errno,
+                      _("Error setting position in file \"%s\":\n\n  %s"),
+                      f->name, _bft_file_error_string(f));
+
+#endif /* SIZEOF_LONG */
   }
 
 #if defined(HAVE_ZLIB)
@@ -1070,7 +1146,7 @@ bft_file_seek(bft_file_t             *const f,
 
     if (retval != 0)
       _bft_file_error(__FILE__, __LINE__, 0,
-                      _("Error setting position in file \"%s\":\n"),
+                      _("Error setting position in file \"%s\":\n\n  %s"),
                       f->name, _bft_file_error_string(f));
   }
 
