@@ -264,12 +264,12 @@ static void _bft_mem_summary(FILE  *f)
 
   _bft_mem_size_val(_bft_mem_global_alloc_cur, value, &unit);
   fprintf(f,
-          _("Theoretical current allocated memory:   %8ld.%ld %cB\n"),
+          _("Theoretical current allocated memory:   %8lu.%lu %cB\n"),
           value[0], value[1], unit);
 
   _bft_mem_size_val(_bft_mem_global_alloc_max, value, &unit);
   fprintf(f,
-          _("Theoretical maximum allocated memory:   %8ld.%ld %cB\n"),
+          _("Theoretical maximum allocated memory:   %8lu.%lu %cB\n"),
           value[0], value[1], unit);
 
   fprintf(f,
