@@ -25,7 +25,6 @@
 #include <string.h>
 
 #include <sys/time.h>
-#include <sys/resource.h>
 #include <unistd.h>
 
 #include "bft_error.h"
@@ -52,8 +51,6 @@ main (int argc, char *argv[])
 
   /* BFT initialization and environment */
 
-  bft_mem_usage_set_options(BFT_MEM_USAGE_TRACK_PR_SIZE |
-                            BFT_MEM_USAGE_TRACK_ALLOC_SIZE);
   bft_mem_usage_init();
 
   bft_mem_init("bft_mem_log_file");
@@ -68,9 +65,6 @@ main (int argc, char *argv[])
   BFT_MALLOC(p2, 100000, double);
   printf("p2 = %p\n", p2);
 
-  bft_mem_usage_set_options(BFT_MEM_USAGE_TRACK_PR_SIZE |
-                            BFT_MEM_USAGE_TRACK_ALLOC_SIZE);
-
   p3 = NULL;
   BFT_REALLOC(p3, 100000, double);
   printf("p3 = %p\n", p3);
@@ -102,8 +96,6 @@ main (int argc, char *argv[])
 
   printf("max memory usage: %lu kB\n",
          (unsigned long) bft_mem_usage_max_pr_size());
-  printf("max memory usage (alloc): %lu kB\n",
-         (unsigned long) bft_mem_usage_max_alloc_size());
 
   BFT_MALLOC(p1, 10000, long);
   printf("p1 = %p\n", p1);
